from django.contrib import admin
from edc_action_item import action_fieldset_tuple
from edc_data_manager.data_manager_modeladmin_mixin import DataManagerModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ...admin_site import meta_prn_admin
from ...forms import EndOfStudyPhaseThreeForm
from ...models import EndOfStudy
from .model_admin_mixin import EndOfStudyAdminMixin


@admin.register(EndOfStudy, site=meta_prn_admin)
class EndOfStudyPhaseThreeAdmin(
    EndOfStudyAdminMixin,
    DataManagerModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):

    form = EndOfStudyPhaseThreeForm

    fieldsets = (
        [
            "Part 1:",
            {
                "fields": (
                    "subject_identifier",
                    "offschedule_datetime",
                    "last_seen_date",
                    "offschedule_reason",
                    "other_offschedule_reason",
                    "ltfu_date",
                    "death_date",
                    "clinical_withdrawal_reason",
                    "clinical_withdrawal_reason_other",
                    "toxicity_withdrawal_reason",
                    "toxicity_withdrawal_reason_other",
                    "comment",
                )
            },
        ],
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "offschedule_reason": admin.VERTICAL,
        "clinical_withdrawal_reason": admin.VERTICAL,
        "toxicity_withdrawal_reason": admin.VERTICAL,
    }



# Generated by Django 3.2.11 on 2022-03-16 18:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0039_auto_20220312_1938'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='converted_ogtt_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='fbg_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='inclusion_a',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with FBG (6.1 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='inclusion_b',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='inclusion_c',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='inclusion_d',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ogtt2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='Repeat OGTT Units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ogtt_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='OGTT Units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='converted_ogtt_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='fbg_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='inclusion_a',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with FBG (6.1 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='inclusion_b',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='inclusion_c',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='inclusion_d',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ogtt2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='Repeat OGTT Units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ogtt_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='OGTT Units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='converted_ogtt_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='fbg_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='inclusion_a',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with FBG (6.1 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='inclusion_b',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='inclusion_c',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='inclusion_d',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ogtt2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='Repeat OGTT Units'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ogtt_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='OGTT Units'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='converted_ogtt_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='fbg_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='inclusion_a',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with FBG (6.1 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='inclusion_b',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='inclusion_c',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='inclusion_d',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ogtt2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='Repeat OGTT Units'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ogtt_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='OGTT Units'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='converted_fbg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat FBG level'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='Repeat OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='converted_ogtt_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='OGTT (2-hours)'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='fbg_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='FBG units'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='inclusion_a',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with FBG (6.1 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='inclusion_b',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='inclusion_c',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='inclusion_d',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('tbd', 'To be determined')], default='tbd', help_text='system calculated', max_length=15, verbose_name='BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ogtt2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='Repeat OGTT Units'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ogtt_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='OGTT Units'),
        ),
    ]



from .end_of_study_form import EndOfStudyPhaseThreeForm, EndOfStudyPhaseTwoForm
from .loss_to_followup_form import LossToFollowupForm
from .pregnancy_notification_form import PregnancyNotificationForm
from .protocol_deviation_violation_form import ProtocolDeviationViolationForm



# Generated by Django 2.2.6 on 2019-11-04 06:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("meta_lists", "0004_auto_20191102_1859")]

    operations = [
        migrations.AlterModelOptions(
            name="arvregimens",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "ARV Regimens",
                "verbose_name_plural": "ARV Regimens",
            },
        ),
        migrations.AlterModelOptions(
            name="baselinesymptoms",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "Baseline Symptoms",
                "verbose_name_plural": "Baseline Symptoms",
            },
        ),
        migrations.AlterModelOptions(
            name="diabetessymptoms",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "Diabetes Symptoms",
                "verbose_name_plural": "Diabetes Symptoms",
            },
        ),
        migrations.AlterModelOptions(
            name="hypertensionmedications",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "Hypertension Medications",
                "verbose_name_plural": "Hypertension Medications",
            },
        ),
        migrations.AlterModelOptions(
            name="nonadherencereasons",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "Non-Adherence Reasons",
                "verbose_name_plural": "Non-Adherence Reasons",
            },
        ),
        migrations.AlterModelOptions(
            name="offstudyreasons",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "Offstudy Reasons",
                "verbose_name_plural": "Offstudy Reasons",
            },
        ),
        migrations.AlterModelOptions(
            name="oiprophylaxis",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "OI Prophylaxis",
                "verbose_name_plural": "OI Prophylaxis",
            },
        ),
        migrations.AlterModelOptions(
            name="symptoms",
            options={
                "ordering": ["display_index", "display_name"],
                "verbose_name": "Symptoms",
                "verbose_name_plural": "Symptoms",
            },
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-28 20:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0053_auto_20210628_2105"),
    ]

    operations = [
        migrations.RenameField(
            model_name="bloodresultslipid",
            old_name="hdl",
            new_name="hdl_value",
        ),
        migrations.RenameField(
            model_name="bloodresultslipid",
            old_name="ldl",
            new_name="ldl_value",
        ),
        migrations.RenameField(
            model_name="bloodresultslipid",
            old_name="trig",
            new_name="trig_value",
        ),
        migrations.RenameField(
            model_name="historicalbloodresultslipid",
            old_name="hdl",
            new_name="hdl_value",
        ),
        migrations.RenameField(
            model_name="historicalbloodresultslipid",
            old_name="ldl",
            new_name="ldl_value",
        ),
        migrations.RenameField(
            model_name="historicalbloodresultslipid",
            old_name="trig",
            new_name="trig_value",
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="hba1c_quantifier",
            field=models.CharField(
                blank=True,
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="hba1c_units",
            field=models.CharField(
                blank=True, default="%", max_length=15, null=True, verbose_name="units"
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="hba1c_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=8,
                null=True,
                verbose_name="HbA1c value",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="hba1c_quantifier",
            field=models.CharField(
                blank=True,
                choices=[
                    ("=", "="),
                    (">", ">"),
                    (">=", ">="),
                    ("<", "<"),
                    ("<=", "<="),
                ],
                default="=",
                max_length=10,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="hba1c_units",
            field=models.CharField(
                blank=True, default="%", max_length=15, null=True, verbose_name="units"
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="hba1c_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=8,
                null=True,
                verbose_name="HbA1c value",
            ),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-26 19:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("meta_lists", "0001_initial")]

    operations = [
        migrations.AlterModelOptions(
            name="arvregimens", options={"ordering": ["display_index", "display_name"]}
        ),
        migrations.AlterModelOptions(
            name="baselinesymptoms",
            options={"ordering": ["display_index", "display_name"]},
        ),
        migrations.AlterModelOptions(
            name="diabetessymptoms",
            options={"ordering": ["display_index", "display_name"]},
        ),
        migrations.AlterModelOptions(
            name="offstudyreasons",
            options={"ordering": ["display_index", "display_name"]},
        ),
        migrations.AlterModelOptions(
            name="oiprophylaxis",
            options={"ordering": ["display_index", "display_name"]},
        ),
        migrations.AlterModelOptions(
            name="symptoms", options={"ordering": ["display_index", "display_name"]}
        ),
        migrations.RemoveIndex(
            model_name="arvregimens", name="meta_lists__id_10dc52_idx"
        ),
        migrations.RemoveIndex(
            model_name="baselinesymptoms", name="meta_lists__id_5d3e73_idx"
        ),
        migrations.RemoveIndex(
            model_name="diabetessymptoms", name="meta_lists__id_de7534_idx"
        ),
        migrations.RemoveIndex(
            model_name="offstudyreasons", name="meta_lists__id_d6cb87_idx"
        ),
        migrations.RemoveIndex(
            model_name="oiprophylaxis", name="meta_lists__id_509dd4_idx"
        ),
        migrations.RemoveIndex(model_name="symptoms", name="meta_lists__id_f12990_idx"),
        migrations.RenameField(
            model_name="arvregimens", old_name="name", new_name="display_name"
        ),
        migrations.RenameField(
            model_name="baselinesymptoms", old_name="name", new_name="display_name"
        ),
        migrations.RenameField(
            model_name="diabetessymptoms", old_name="name", new_name="display_name"
        ),
        migrations.RenameField(
            model_name="offstudyreasons", old_name="name", new_name="display_name"
        ),
        migrations.RenameField(
            model_name="oiprophylaxis", old_name="name", new_name="display_name"
        ),
        migrations.RenameField(
            model_name="symptoms", old_name="name", new_name="display_name"
        ),
        migrations.RenameField(
            model_name="arvregimens", old_name="short_name", new_name="name"
        ),
        migrations.RenameField(
            model_name="baselinesymptoms", old_name="short_name", new_name="name"
        ),
        migrations.RenameField(
            model_name="diabetessymptoms", old_name="short_name", new_name="name"
        ),
        migrations.RenameField(
            model_name="offstudyreasons", old_name="short_name", new_name="name"
        ),
        migrations.RenameField(
            model_name="oiprophylaxis", old_name="short_name", new_name="name"
        ),
        migrations.RenameField(
            model_name="symptoms", old_name="short_name", new_name="name"
        ),
        migrations.AddIndex(
            model_name="arvregimens",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="meta_lists__id_18210c_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="baselinesymptoms",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="meta_lists__id_5cfb01_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="diabetessymptoms",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="meta_lists__id_9b6cbb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="offstudyreasons",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="meta_lists__id_713d23_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="oiprophylaxis",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="meta_lists__id_399299_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="symptoms",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="meta_lists__id_1f5296_idx",
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-03-04 02:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0031_auto_20220304_0448'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ifg_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ifg_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ifg_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ifg_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ifg_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ifg_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ifg_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ifg_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ifg_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ifg_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ifg_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ifg_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ifg_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ifg_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ifg_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
    ]



from django import forms
from django.conf import settings
from edc_consent.form_validators import SubjectConsentFormValidatorMixin
from edc_consent.modelform_mixins import ConsentModelFormMixin
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from ..models import SubjectConsent

# TODO: Assessment score "The client has completed the
#  assessment of understanding with a passing score"


class SubjectConsentFormValidator(SubjectConsentFormValidatorMixin, FormValidator):
    def validate_identity(self) -> None:
        """Override to validate `identity_type` is a hospital
        number and `identity` matches the screening form.
        """
        if self.cleaned_data.get("identity_type") != "hospital_no":
            raise forms.ValidationError(
                {"identity_type": "Expected 'hospital number'."}
            )

        if self.subject_screening.hospital_identifier != self.cleaned_data.get(
            "identity"
        ):
            raise forms.ValidationError(
                {
                    "identity": (
                        "The hospital identifier does not match that "
                        "reported at screening."
                    )
                }
            )

    def validate_consent_datetime(self):
        if getattr(settings, "EDC_CONSENT_BYPASS_CONSENT_DATETIME_VALIDATION", False):
            return None
        return super().validate_consent_datetime()


class SubjectConsentForm(
    SiteModelFormMixin, FormValidatorMixin, ConsentModelFormMixin, forms.ModelForm
):

    form_validator_cls = SubjectConsentFormValidator

    screening_identifier = forms.CharField(
        label="Screening identifier",
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean_gender_of_consent(self):
        return None

    def clean_guardian_and_dob(self):
        return None

    def clean_identity_with_unique_fields(self):
        return None

    class Meta:
        model = SubjectConsent
        fields = "__all__"
        help_texts = {
            "identity": (
                "Use Country ID Number, Passport number, driver's license "
                "number or Country ID receipt number"
            ),
            "witness_name": (
                "Required only if participant is illiterate. "
                "Format is 'LASTNAME, FIRSTNAME'. "
                "All uppercase separated by a comma."
            ),
        }



from edc_blood_results import BLOOD_RESULTS_RFT_ACTION
from edc_blood_results.model_mixins import (
    BloodResultsModelMixin,
    CreatinineModelMixin,
    EgfrModelMixin,
    UreaModelMixin,
    UricAcidModelMixin,
)
from edc_crf.crf_with_action_model_mixin import CrfWithActionModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.panels import rft_panel
from edc_model import models as edc_models
from edc_reportable import calculate_egfr
from edc_reportable.units import EGFR_UNITS
from edc_screening.utils import get_subject_screening_model_cls


class BloodResultsRft(
    CrfWithActionModelMixin,
    CreatinineModelMixin,
    EgfrModelMixin,
    UreaModelMixin,
    UricAcidModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    edc_models.BaseUuidModel,
):
    action_name = BLOOD_RESULTS_RFT_ACTION
    tracking_identifier_prefix = "RF"
    lab_panel = rft_panel

    def save(self, *args, **kwargs):
        if self.creatinine_value:
            subject_screening = get_subject_screening_model_cls().objects.get(
                subject_identifier=self.subject_visit.subject_identifier
            )
            self.egfr_value = calculate_egfr(
                gender=subject_screening.gender,
                ethnicity=subject_screening.ethnicity,
                age_in_years=subject_screening.age_in_years,
                creatinine_value=self.creatinine_value,
                creatinine_units=self.creatinine_units,
            )
            self.egfr_units = EGFR_UNITS
        super().save(*args, **kwargs)

    class Meta(CrfWithActionModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Blood Result: RFT"
        verbose_name_plural = "Blood Results: RFT"



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin

from ..form_validators import ScreeningPartTwoFormValidator
from ..models import ScreeningPartTwo
from .field_lists import get_part_two_fields


class ScreeningPartTwoForm(
    AlreadyConsentedFormMixin, FormValidatorMixin, forms.ModelForm
):

    form_validator_cls = ScreeningPartTwoFormValidator

    AUTO_NUMBER_START = 17

    def clean(self):
        cleaned_data = super().clean()
        return cleaned_data

    class Meta:
        model = ScreeningPartTwo
        fields = get_part_two_fields()



from django.core.management.base import BaseCommand, CommandError
from edc_pharmacy.models import Medication, Rx
from edc_pharmacy.utils import create_prescription

from meta_consent.models import SubjectConsent
from meta_edc.meta_version import get_meta_version
from meta_pharmacy.constants import METFORMIN


class Command(BaseCommand):
    help = "Create missing prescriptions"

    def handle(self, *args, **options):
        medication = Medication.objects.get(name=METFORMIN)
        subject_identifiers = Rx.objects.values_list(
            "subject_identifier", flat=True
        ).all()
        for instance in SubjectConsent.objects.exclude(
            subject_identifier__in=subject_identifiers
        ):
            create_prescription(
                subject_identifier=instance.subject_identifier,
                report_datetime=instance.consent_datetime,
                randomizer_name=get_meta_version(),
                medication_name=medication.name,
            )



from django import forms
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator
from edc_he.form_validators import HeEducationFormValidatorMixin

from ..models import HealthEconomicsSimple


class HealthEconomicsFormValidator(
    HeEducationFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    def clean(self):
        self.clean_education()


class HealthEconomicsSimpleForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HealthEconomicsFormValidator

    class Meta:
        model = HealthEconomicsSimple
        fields = "__all__"



from edc_lab_panel.panels import (
    blood_glucose_panel,
    fbc_panel,
    hba1c_panel,
    insulin_panel,
    lft_panel,
    lipids_panel,
    rft_panel,
)
from edc_visit_schedule import FormsCollection, Requisition

requisitions_prn = FormsCollection(
    Requisition(
        show_order=200, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=220, panel=hba1c_panel, required=True, additional=False),
    Requisition(show_order=230, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=240, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=250, panel=fbc_panel, required=True, additional=False),
    Requisition(show_order=260, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=270, panel=insulin_panel, required=True, additional=False),
    name="requisitions_prn",
)

requisitions_unscheduled = FormsCollection(
    Requisition(
        show_order=200, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_unscheduled",
)

requisitions_d1 = FormsCollection(
    Requisition(show_order=30, panel=fbc_panel, required=True, additional=False),
    Requisition(show_order=40, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=60, panel=insulin_panel, required=True, additional=False),
    name="requisitions_day1",
)

requisitions_w2 = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_week2",
)

requisitions_1m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_month1",
)

requisitions_3m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=20, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=30, panel=lft_panel, required=True, additional=False),
    name="requisitions_month3",
)

requisitions_6m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=20, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=30, panel=lft_panel, required=True, additional=False),
    name="requisitions_month6",
)

requisitions_9m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_month9",
)

requisitions_12m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=30, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=60, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month12",
)

requisitions_15m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_month15",
)

requisitions_18m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=20, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month18",
)

requisitions_21m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_month21",
)

requisitions_24m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=30, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=60, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month24",
)

requisitions_27m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_month27",
)

requisitions_30m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=20, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=30, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month30",
)

requisitions_33m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    name="requisitions_month33",
)

requisitions_36m = FormsCollection(
    Requisition(
        show_order=10, panel=blood_glucose_panel, required=True, additional=False
    ),
    Requisition(show_order=30, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=40, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=50, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=60, panel=fbc_panel, required=True, additional=False),
    name="requisitions_month36",
)



from edc_constants.constants import NO, NOT_APPLICABLE, PATIENT, POS, YES
from edc_utils import get_utcnow
from edc_visit_tracking.constants import SCHEDULED
from faker import Faker
from model_bakery.recipe import Recipe

from meta_ae.constants import HOSPITAL_CLINIC

from .constants import LIVE_AT_TERM, NO_COMPLICATIONS
from .models import (
    BirthOutcomes,
    Delivery,
    FollowupExamination,
    SubjectRequisition,
    SubjectVisit,
    UrinePregnancy,
)

fake = Faker()

subjectvisit = Recipe(SubjectVisit, reason=SCHEDULED)

subjectrequisition = Recipe(SubjectRequisition)

urinepregnancy = Recipe(
    UrinePregnancy,
    action_identifier=None,
    report_datetime=get_utcnow(),
    performed=YES,
    not_performed_reason=None,
    bhcg_value=POS,
    notified=False,
    notified_datetime=None,
)

followupexamination = Recipe(
    FollowupExamination,
    # site=None,
    # report_datetime=None,
    # subject_visit=None,
    # symptoms_detail=None,
    # attended_clinic=None,
    # admitted_hospital=None,
    # attended_clinic_detail=None,
    # prescribed_medication=None,
    # prescribed_medication_detail=None,
    # attended_clinic_sae=None,
    # any_other_problems=None,
    # any_other_problems_detail=None,
    # any_other_problems_sae=None,
    # any_other_problems_sae_grade=None,
    # art_change=None,
    # art_change_reason=None,
    # art_new_regimen_other=None,
    # abdominal_tenderness=None,
    # enlarged_liver=None,
    # jaundice=None,
    # comment=None,
    # lactic_acidosis=None,
    # hepatomegaly=None,
    # referral=None,
    # referral_reason=None,
)

delivery = Recipe(
    Delivery,
    # site=None,
    action_identifier=None,
    report_datetime=get_utcnow(),
    info_available=YES,
    info_source=PATIENT,
    informant_relation=NOT_APPLICABLE,
    informant_relation_other=None,
    delivery_datetime=get_utcnow(),
    delivery_time_estimated=NO,
    delivery_location=HOSPITAL_CLINIC,
    delivery_location_other=None,
    delivery_location_name="Big hospital",
    delivery_ga=40,
    gm_treated=NO,
    maternal_outcome=NO_COMPLICATIONS,
)

birthoutcomes = Recipe(
    BirthOutcomes,
    delivery=None,
    action_identifier=None,
    report_datetime=get_utcnow(),
    birth_order=1,
    birth_outcome=LIVE_AT_TERM,
    birth_weight=320,
)



from django.db import models
from edc_model import models as edc_models
from edc_vitals.model_mixins import (
    BloodPressureModelMixin,
    SimpleBloodPressureModelMixin,
)

from ..choices import WEIGHT_DETERMINATION
from ..model_mixins import CrfModelMixin, VitalsFieldsModelMixin


class FollowupVitals(
    VitalsFieldsModelMixin,
    BloodPressureModelMixin,
    SimpleBloodPressureModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):

    # TODO: Grading for blood pressure?? sokoine See DAIDS
    # TODO: Add action item modelmixin, grading like blood results

    weight_determination = models.CharField(
        verbose_name="Is weight estimated or measured?",
        max_length=15,
        choices=WEIGHT_DETERMINATION,
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Clinic follow up: Vitals"
        verbose_name_plural = "Clinic follow ups: Vitals"



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_action_item.forms import ActionItemFormMixin
from edc_constants.constants import NO, OTHER, YES
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator

from meta_ae.constants import HOSPITAL_CLINIC
from meta_prn.models import PregnancyNotification

from ..models import Delivery


class DeliveryFormValidator(FormValidator):
    def clean(self):
        try:
            PregnancyNotification.objects.get(
                subject_identifier=self.cleaned_data.get(
                    "subject_visit"
                ).subject_identifier
            )
        except ObjectDoesNotExist:
            raise forms.ValidationError(
                f"{PregnancyNotification._meta.verbose_name} not found."
            )

        self.required_if(
            NO, field="info_available", field_required="info_not_available_reason"
        )
        self.required_if(
            YES, field="info_available", field_required="info_not_available_reason"
        )

        self.validate_informant()

        self.validate_delivery_location()

    def validate_informant(self):
        self.required_if(OTHER, field="info_source", field_required="info_source_other")
        self.required_if(NO, field="info_source", field_required="informant_relation")

        self.validate_other_specify(
            field="informant_relation", field_required="informant_relation_other"
        )

    def validate_delivery_location(self):
        self.validate_other_specify(
            field="delivery_location", field_required="delivery_location_other"
        )
        self.required_if(
            HOSPITAL_CLINIC,
            field="delivery_location",
            field_required="delivery_location_name",
        )
        self.validate_other_specify(
            field="delivery_location", field_required="delivery_location_other"
        )


class DeliveryForm(CrfModelFormMixin, ActionItemFormMixin, forms.ModelForm):

    form_validator_cls = DeliveryFormValidator

    class Meta:
        model = Delivery
        fields = "__all__"



# Generated by Django 2.2.7 on 2019-11-14 05:21

import edc_model_fields.fields.other_charfield
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_ae", "0003_auto_20191102_0033"),
    ]

    operations = [
        migrations.AlterField(
            model_name="deathreport",
            name="cause_of_death_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=100,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="cause_of_death_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=100,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



from edc_adherence.model_mixins import MedicationAdherenceModelMixin
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class MedicationAdherence(
    MedicationAdherenceModelMixin, CrfModelMixin, edc_models.BaseUuidModel
):
    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Medication Adherence"
        verbose_name_plural = "Medication Adherence"



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin
from edc_pharmacy.admin import DosageGuidelineAdmin as BaseDosageGuidelineAdmin
from edc_pharmacy.admin import FormulationAdmin as BaseFormulationAdmin
from edc_pharmacy.admin import MedicationAdmin as BaseMedicationAdmin
from edc_pharmacy.models import DosageGuideline, Formulation, Medication

from meta_lists.models import ArvRegimens

from ..admin_site import meta_subject_admin


@admin.register(ArvRegimens, site=meta_subject_admin)
class ArvRegimensAdmin(ListModelAdminMixin, admin.ModelAdmin):
    """Registered again for the autocomplete field"""

    pass


@admin.register(DosageGuideline, site=meta_subject_admin)
class DosageGuidelineAdmin(BaseDosageGuidelineAdmin):
    """Registered again for the autocomplete field"""

    pass


@admin.register(Medication, site=meta_subject_admin)
class MedicationAdmin(BaseMedicationAdmin):
    """Registered again for the autocomplete field"""

    pass


@admin.register(Formulation, site=meta_subject_admin)
class FormulationAdmin(BaseFormulationAdmin):
    """Registered again for the autocomplete field"""

    pass



from edc_consent.consent import Consent
from edc_consent.site_consents import site_consents
from edc_constants.constants import FEMALE, MALE
from edc_protocol import Protocol

v1 = Consent(
    "meta_consent.subjectconsent",
    version="1",
    start=Protocol().study_open_datetime,
    end=Protocol().study_close_datetime,
    age_min=18,
    age_is_adult=18,
    age_max=110,
    gender=[MALE, FEMALE],
)

site_consents.register(v1)



from django.utils.safestring import mark_safe
from edc_action_item.action_with_notification import ActionWithNotification
from edc_action_item.site_action_items import site_action_items
from edc_adverse_event.constants import DEATH_REPORT_ACTION
from edc_constants.constants import HIGH_PRIORITY, TBD, YES
from edc_ltfu.constants import LTFU_ACTION
from edc_offstudy.constants import END_OF_STUDY_ACTION
from edc_protocol_violation.action_items import (
    ProtocolDeviationViolationAction as BaseProtocolDeviationViolationAction,
)

from meta_subject.constants import DELIVERY_ACTION, URINE_PREGNANCY_ACTION

from .constants import (
    OFFSCHEDULE_ACTION,
    OFFSCHEDULE_POSTNATAL_ACTION,
    OFFSCHEDULE_PREGNANCY_ACTION,
    PREGNANCY_NOTIFICATION_ACTION,
    UNBLINDING_REQUEST_ACTION,
    UNBLINDING_REVIEW_ACTION,
)
from .pregnancy_mixin import PregnancyMixin


class OffscheduleAction(ActionWithNotification):
    name = OFFSCHEDULE_ACTION
    display_name = "Submit Off-Schedule"
    notification_display_name = "Off-Schedule"
    parent_action_names = [
        UNBLINDING_REVIEW_ACTION,
        DEATH_REPORT_ACTION,
        LTFU_ACTION,
    ]
    reference_model = "meta_prn.offschedule"
    show_link_to_changelist = True
    admin_site_name = "meta_prn_admin"
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = [END_OF_STUDY_ACTION]
        return next_actions


class OffschedulePregnancyAction(ActionWithNotification):
    name = OFFSCHEDULE_PREGNANCY_ACTION
    display_name = "Submit Off-Schedule (Pregnancy)"
    notification_display_name = "Off-Schedule (Pregnancy)"
    parent_action_names = [
        UNBLINDING_REVIEW_ACTION,
        DEATH_REPORT_ACTION,
        LTFU_ACTION,
        DELIVERY_ACTION,
    ]
    reference_model = "meta_prn.offschedulepregnancy"
    show_link_to_changelist = True
    admin_site_name = "meta_prn_admin"
    priority = HIGH_PRIORITY


class OffschedulePostnatalAction(ActionWithNotification):
    name = OFFSCHEDULE_POSTNATAL_ACTION
    display_name = "Submit Off-Schedule (Postnatal)"
    notification_display_name = "Off-Schedule (Postnatal)"
    parent_action_names = [
        UNBLINDING_REVIEW_ACTION,
        DEATH_REPORT_ACTION,
        LTFU_ACTION,
        DELIVERY_ACTION,
    ]
    reference_model = "meta_prn.offschedulepostnatal"
    show_link_to_changelist = True
    admin_site_name = "meta_prn_admin"
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = [END_OF_STUDY_ACTION]
        return next_actions


class EndOfStudyAction(ActionWithNotification):
    name = END_OF_STUDY_ACTION
    display_name = "Submit End of Study Report"
    notification_display_name = "End of Study Report"
    parent_action_names = [
        OFFSCHEDULE_ACTION,
        OFFSCHEDULE_PREGNANCY_ACTION,
    ]
    reference_model = "meta_prn.endofstudy"
    show_link_to_changelist = True
    admin_site_name = "meta_prn_admin"
    priority = HIGH_PRIORITY


class LossToFollowupAction(PregnancyMixin, ActionWithNotification):
    name = LTFU_ACTION
    display_name = "Submit Loss to Follow Up Report"
    notification_display_name = " Loss to Follow Up Report"
    parent_action_names = []
    reference_model = "meta_prn.losstofollowup"
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = "meta_prn_admin"
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = [self.get_next_offschedule_action()]
        return next_actions


class PregnancyNotificationAction(ActionWithNotification):
    name = PREGNANCY_NOTIFICATION_ACTION
    display_name = "Submit Pregnancy Notification"
    notification_display_name = "Pregnancy Notification"
    parent_action_names = [URINE_PREGNANCY_ACTION]
    reference_model = "meta_prn.pregnancynotification"
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = "meta_prn_admin"
    priority = HIGH_PRIORITY


# TODO: WithdrawalStudyMedicationAction
# class WithdrawalStudyMedicationAction(ActionWithNotification):
#     name = WITHDRAWAL_STUDY_MEDICATION_ACTION


class UnblindingRequestAction(ActionWithNotification):
    name = UNBLINDING_REQUEST_ACTION
    display_name = "Unblinding request"
    notification_display_name = " Unblinding request"
    parent_action_names = []
    reference_model = "edc_unblinding.unblindingrequest"
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = "edc_unblinding_admin"
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = []
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=UNBLINDING_REVIEW_ACTION,
            required=self.reference_obj.approved == TBD,
        )
        return next_actions


class UnblindingReviewAction(PregnancyMixin, ActionWithNotification):
    name = UNBLINDING_REVIEW_ACTION
    display_name = "Unblinding review pending"
    notification_display_name = " Unblinding review needed"
    parent_action_names = [UNBLINDING_REQUEST_ACTION]
    reference_model = "edc_unblinding.unblindingreview"
    show_link_to_changelist = True
    admin_site_name = "edc_unblinding_admin"
    priority = HIGH_PRIORITY
    color_style = "info"
    create_by_user = False
    instructions = mark_safe(
        "This report is to be completed by the UNBLINDING REVIEWERS only."
    )

    def get_next_actions(self):
        next_actions = []
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=self.get_next_offschedule_action(),
            required=self.reference_obj.approved == YES,
        )
        return next_actions


class ProtocolDeviationViolationAction(BaseProtocolDeviationViolationAction):
    reference_model = "meta_prn.protocoldeviationviolation"
    admin_site_name = "meta_prn_admin"


site_action_items.register(EndOfStudyAction)
site_action_items.register(LossToFollowupAction)
site_action_items.register(PregnancyNotificationAction)
site_action_items.register(ProtocolDeviationViolationAction)
site_action_items.register(UnblindingRequestAction)
site_action_items.register(UnblindingReviewAction)
site_action_items.register(OffscheduleAction)
site_action_items.register(OffschedulePregnancyAction)
site_action_items.register(OffschedulePostnatalAction)



# Generated by Django 3.2.4 on 2021-08-04 01:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0069_auto_20210801_2021"),
    ]

    operations = [
        migrations.AlterField(
            model_name="glucose",
            name="ifg_units",
            field=models.CharField(
                choices=[
                    ("mg/dL", "mg/dL"),
                    ("mmol/L", "mmol/L (millimoles/L)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="IFG units",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="ogtt_units",
            field=models.CharField(
                choices=[
                    ("mg/dL", "mg/dL"),
                    ("mmol/L", "mmol/L (millimoles/L)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="ifg_units",
            field=models.CharField(
                choices=[
                    ("mg/dL", "mg/dL"),
                    ("mmol/L", "mmol/L (millimoles/L)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="IFG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="ogtt_units",
            field=models.CharField(
                choices=[
                    ("mg/dL", "mg/dL"),
                    ("mmol/L", "mmol/L (millimoles/L)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatienthistory",
            name="on_hypertension_treatment",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Is the patient on treatment for hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="patienthistory",
            name="on_hypertension_treatment",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Is the patient on treatment for hypertension?",
            ),
        ),
    ]



from edc_constants.constants import (
    ABSENT,
    DEAD,
    MICROSCOPY,
    NO,
    NO_EXAM,
    NOT_APPLICABLE,
    OTHER,
    PATIENT,
    PRESENT,
    PRESENT_WITH_REINFORCEMENT,
    RAPID_TEST,
    YES,
)
from edc_reportable.constants import GRADE3, GRADE4
from edc_visit_tracking.constants import MISSED_VISIT, SCHEDULED, UNSCHEDULED

from meta_ae.choices import INFORMANT_RELATIONSHIP
from meta_ae.constants import HOSPITAL_CLINIC

from .constants import (
    ALL_OF_THE_TIME,
    APPT,
    APPT_OTHER,
    GOOD_BIT_OF_THE_TIME,
    LITTLE_OF_THE_TIME,
    LIVE_AT_TERM,
    LIVE_PRETERM,
    MOST_OF_THE_TIME,
    NO_COMPLICATIONS,
    NONE_OF_THE_TIME,
    SOME_OF_THE_TIME,
)

ACTIVITY_CHOICES = (
    ("working", "Working"),
    ("studying", "Studying"),
    ("caring_for_children", "Caring for children"),
    (OTHER, "Other, please specify"),
)


CHILDCARE_CHOICES = (
    (NOT_APPLICABLE, "Not applicable"),
    ("working", "Working"),
    ("studying", "Studying"),
    ("caring_for_children", "Caring for children"),
    ("house_maintenance", "House maintenance"),
    ("nothing", "Nothing"),
    (OTHER, "Other, specify"),
)

DELIVERY_INFO_SOURCE = (
    (PATIENT, "Study participant"),
    (OTHER, "Other"),
    (NOT_APPLICABLE, "Not applicable"),
)

DELIVERY_LOCATIONS = (
    ("home", "At home"),
    (HOSPITAL_CLINIC, "Hospital / Clinic"),
    (OTHER, "Other location, specify"),
    (NOT_APPLICABLE, "Not applicable"),
)

DESCRIBE_HEALTH_CHOICES = (
    ("excellent", "Excellent"),
    ("very_good", "Very good"),
    ("good", "Good"),
    ("fair", "Fair"),
    ("poor", "Poor"),
)

DYSLIPIDAEMIA_RX_CHOICES = (
    ("atorvastatin", "Atorvastatin"),
    ("rosuvastatin", "Rosuvastatin"),
    (OTHER, "Other, specify below ..."),
    (NOT_APPLICABLE, "Not applicable"),
)

FEELING_DURATION_CHOICES = (
    (ALL_OF_THE_TIME, "All of the time"),
    (MOST_OF_THE_TIME, "Most of the time"),
    (GOOD_BIT_OF_THE_TIME, " A good bit of the time"),
    (SOME_OF_THE_TIME, "Some of the time"),
    (LITTLE_OF_THE_TIME, "A little of the time"),
    (NONE_OF_THE_TIME, "None of the time"),
)


FETAL_OUTCOMES = (
    (LIVE_AT_TERM, "Live birth at term"),
    (LIVE_PRETERM, "Live birth preterm"),
    ("stillbirth", "Stillbirth"),
    ("miscarriage", "Miscarriage"),
)

FOLLOWUP_REASONS = (
    (APPT, "Study appointment"),
    (APPT_OTHER, "Other routine appointment"),
    (UNSCHEDULED, "Unschedule visit"),
    (OTHER, "Other reason, specify below ..."),
)

GRADE34_CHOICES = (
    (GRADE3, "Grade 3"),
    (GRADE4, "Grade 4"),
    (NOT_APPLICABLE, "Not applicable"),
)

HEALTH_LIMITED_CHOICES = (
    ("limited_a_lot", "YES, limited a lot"),
    ("limited_a_little", "YES, limited a little"),
    ("not_limited_at_all", "NO, not at all limited"),
)

INFO_SOURCE = (
    ("hospital_notes", "Hospital notes"),
    ("outpatient_cards", "Outpatient cards"),
    ("patient", "Patient"),
    ("collateral_history", "Collateral History from relative/guardian"),
    (NOT_APPLICABLE, "Not applicable (if missed)"),
    (OTHER, "Other"),
)

DELIVERY_INFORMANT_RELATIONSHIP = list(INFORMANT_RELATIONSHIP)
DELIVERY_INFORMANT_RELATIONSHIP.extend(
    [
        (HOSPITAL_CLINIC, "Hospital / Clinic records"),
        (NOT_APPLICABLE, "Not applicable"),
    ]
)
DELIVERY_INFORMANT_RELATIONSHIP = tuple(DELIVERY_INFORMANT_RELATIONSHIP)

INTERFERENCE_DURATION_CHOICES = (
    (ALL_OF_THE_TIME, "All of the time"),
    (MOST_OF_THE_TIME, "Most of the time"),
    (SOME_OF_THE_TIME, "Some of the time"),
    (LITTLE_OF_THE_TIME, "A little of the time"),
    (NONE_OF_THE_TIME, "None of the time"),
)

FUNDOSCOPY_CHOICES = (
    ("no_retinopathy", "No retinopathy"),
    ("background_retinopathy", "Background retinopathy"),
    ("pre_proliferative_retinopathy", "Pre-proliferative retinopathy"),
    ("proliferative_retinopathy", "Proliferative retinopathy"),
    ("maculopathy", "Maculopathy"),
    (NO_EXAM, "Exam not performed"),
)

MALARIA_TEST_CHOICES = (
    (RAPID_TEST, "Rapid test"),
    (MICROSCOPY, "Microscopy"),
    (NOT_APPLICABLE, "Not applicable"),
)

MATERNAL_OUTCOMES = (
    (NO_COMPLICATIONS, "No complications"),
    ("complications_full_recovery", "Complications with full recovery"),
    ("complications_ongoing_recovery", "Complications with ongoing recovery"),
    (DEAD, "Maternal mortality"),
    (NOT_APPLICABLE, "Not applicable"),
)

PAYEE_CHOICES = (
    ("own_cash", "Own cash"),
    ("insurance", "Insurance"),
    ("relative", "Relative of others paying"),
    ("free", "Free drugs from the pharmacy"),
    (NOT_APPLICABLE, "Not applicable"),
)

PRESENT_ABSENT_NOEXAM = (
    (PRESENT, "Present"),
    (ABSENT, "Absent"),
    (NO_EXAM, "Exam not performed"),
)

PRESENT_ABSENT_NOEXAM_NDS = (
    (PRESENT, "Present"),
    (PRESENT_WITH_REINFORCEMENT, "Present with reinforcement"),
    (ABSENT, "Absent"),
    (NO_EXAM, "Exam not performed"),
)
# 0 = Present   1 = Present with reinforcement   2 = Absent

TRANSPORT_CHOICES = (
    ("bus", "Bus"),
    ("train", "Train"),
    ("ambulance", "Ambulance"),
    ("private_taxi", "Private taxi"),
    ("own_bicycle", "Own bicycle"),
    ("hired_motorbike", "Hired motorbike"),
    ("own_car", "Own car"),
    ("own_motorbike", "Own motorbike"),
    ("hired_bicycle", "Hired bicycle"),
    ("foot", "Foot"),
    (OTHER, "Other, specify"),
)


VISIT_UNSCHEDULED_REASON = (
    ("patient_unwell_outpatient", "Patient unwell (outpatient)"),
    ("patient_hospitalised", "Patient hospitalised"),
    ("routine_non_study", "Routine appointment (non-study)"),
    ("recurrence_symptoms", "Recurrence of symptoms"),
    (OTHER, "Other"),
    (NOT_APPLICABLE, "Not applicable"),
)

VISIT_REASON = (
    (SCHEDULED, "Scheduled visit"),
    (UNSCHEDULED, "Unscheduled visit"),
    (MISSED_VISIT, "Missed visit"),
)

WEIGHT_DETERMINATION = (("estimated", "Estimated"), ("measured", "Measured"))

WORK_PAIN_INTERFERENCE_CHOICES = (
    ("not_at_all", "Not at all"),
    ("a_little_bit", "A little bit"),
    ("moderately", "Moderately"),
    ("quite_a-bit", "Quite a bit"),
    ("extremely", "Extremely"),
)

YES_NO_NO_EXAM = (
    (YES, YES),
    (NO, NO),
    (NO_EXAM, "Exam not performed"),
)



from edc_constants.constants import NOT_APPLICABLE, OTHER

list_data = {
    "edc_protocol_violation.protocolviolations": [
        ("failure_to_obtain_informed_consent", "Failure to obtain informed " "consent"),
        ("enrollment_of_ineligible_patient", "Enrollment of ineligible patient"),
        (
            "screening_procedure_not_done",
            "Screening procedure required by protocol not done",
        ),
        (
            "screening_or_on_study_procedure",
            "Screening or on-study procedure/lab work required not done",
        ),
        (
            "incorrect_research_treatment",
            "Incorrect research treatment given to patient",
        ),
        (
            "procedure_not_completed",
            "On-study procedure required by protocol not completed",
        ),
        ("visit_non_compliance", "Visit non-compliance"),
        ("medication_stopped_early", "Medication stopped early"),
        ("medication_noncompliance", "Medication_noncompliance"),
        (
            "national_regulations_not_met",
            "Standard WPD, ICH-GCP, local/national regulations not met",
        ),
        (OTHER, "Other"),
        (NOT_APPLICABLE, "Not applicable"),
    ],
}



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_adverse_event.forms import validate_ae_initial_outcome_date
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_registration.modelform_mixins import ModelFormSubjectIdentifierMixin


class AeReviewFormValidator(FormValidator):
    def clean(self):
        pass


class AeReviewModelFormMixin(
    ModelFormSubjectIdentifierMixin,
    ActionItemFormMixin,
    FormValidatorMixin,
):
    form_validator_cls = AeReviewFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean(self):
        cleaned_data = super().clean()
        validate_ae_initial_outcome_date(self)
        return cleaned_data



# Generated by Django 3.2.11 on 2022-03-10 01:39

from django.db import migrations, models
import edc_model.models.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ('meta_prn', '0019_auto_20220309_2230'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalpregnancynotification',
            name='may_contact',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], default='Yes', help_text='If YES/UNSURE, a visit will be scheduled on the EDD. If NO, the participant will be taken off study now', max_length=15, verbose_name='Has the participant agreed to be contacted to provide information on the delivery and birth outcomes?'),
        ),
        migrations.AddField(
            model_name='pregnancynotification',
            name='may_contact',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], default='Yes', help_text='If YES/UNSURE, a visit will be scheduled on the EDD. If NO, the participant will be taken off study now', max_length=15, verbose_name='Has the participant agreed to be contacted to provide information on the delivery and birth outcomes?'),
        ),
        migrations.AlterField(
            model_name='historicalpregnancynotification',
            name='edd',
            field=models.DateField(validators=[edc_model.models.validators.date.date_is_future], verbose_name='Estimated date of delivery'),
        ),
        migrations.AlterField(
            model_name='pregnancynotification',
            name='edd',
            field=models.DateField(validators=[edc_model.models.validators.date.date_is_future], verbose_name='Estimated date of delivery'),
        ),
    ]



from edc_constants.constants import FEMALE, MALE
from edc_randomization import Randomizer
from edc_randomization.randomization_list_importer import (
    RandomizationListImporter,
    RandomizationListImportError,
)
from edc_randomization.site_randomizers import site_randomizers

from meta_edc.meta_version import PHASE_THREE


class RandomizationListImporterPhaseThree(RandomizationListImporter):
    """A class to import the Phase Three randomization list CSV"""

    csv_gender_column = "gender"

    def get_extra_import_options(self, row):
        return dict(gender=self.validate_gender(row))

    def validate_gender(self, row):
        if row[self.csv_gender_column] not in [MALE, FEMALE]:
            raise RandomizationListImportError(
                f"Invalid value for gender. Got {row[self.csv_gender_column]}."
            )
        return row[self.csv_gender_column]


class RandomizerPhaseThree(Randomizer):

    name = PHASE_THREE
    model = "meta_rando.randomizationlist"
    filename = "randomization_list_phase_three.csv"
    importer_cls = RandomizationListImporterPhaseThree

    def __init__(self, gender=None, **kwargs):
        self.gender = gender
        super().__init__(**kwargs)

    @property
    def extra_required_instance_attrs(self):
        """Returns a dict of extra attributes that must have
        value on self.
        """
        return dict(gender=self.gender)

    @property
    def extra_model_obj_options(self):
        """Returns a dict of extra key/value pair for filtering the
        "rando" model.
        """
        return dict(gender=self.gender)

    @classmethod
    def get_extra_list_display(cls):
        """Returns a list of tuples of (pos, field name) for ModelAdmin"""
        return [(4, "gender")]


site_randomizers.register(RandomizerPhaseThree)



# Generated by Django 3.2.4 on 2021-07-09 17:56

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0058_auto_20210702_0533"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mch_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mch_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mch_units",
            field=models.CharField(
                blank=True,
                choices=[("pg/cell", "pg/cell")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mch_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="MCH",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mchc_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mchc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mchc_units",
            field=models.CharField(
                blank=True,
                choices=[("g/dL", "g/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mchc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="MCHC",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mcv_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mcv_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mcv_units",
            field=models.CharField(
                blank=True,
                choices=[("fL/cell", "fL/cell")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="mcv_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="MCV",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="chol_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="chol_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="chol_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultslipid",
            name="chol_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(999),
                ],
                verbose_name="chol",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mch_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mch_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mch_units",
            field=models.CharField(
                blank=True,
                choices=[("pg/cell", "pg/cell")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mch_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="MCH",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mchc_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mchc_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mchc_units",
            field=models.CharField(
                blank=True,
                choices=[("g/dL", "g/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mchc_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="MCHC",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mcv_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mcv_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mcv_units",
            field=models.CharField(
                blank=True,
                choices=[("fL/cell", "fL/cell")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="mcv_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="MCV",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="chol_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="chol_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="chol_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipid",
            name="chol_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(999),
                ],
                verbose_name="chol",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="fasting",
            field=models.CharField(
                choices=[("fasting", "Fasting"), ("non_fasting", "Non-fasting")],
                max_length=25,
                null=True,
                verbose_name="Was this fasting or non-fasting?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="fasting",
            field=models.CharField(
                choices=[("fasting", "Fasting"), ("non_fasting", "Non-fasting")],
                max_length=25,
                null=True,
                verbose_name="Was this fasting or non-fasting?",
            ),
        ),
    ]



from edc_action_item.models import ActionModelMixin
from edc_identifier.model_mixins import TrackingModelMixin
from edc_model.models import BaseUuidModel
from edc_visit_schedule.model_mixins import OffScheduleModelMixin

from meta_prn.constants import (
    OFFSCHEDULE_ACTION,
    OFFSCHEDULE_POSTNATAL_ACTION,
    OFFSCHEDULE_PREGNANCY_ACTION,
)


class OffSchedule(
    ActionModelMixin, TrackingModelMixin, OffScheduleModelMixin, BaseUuidModel
):

    action_name = OFFSCHEDULE_ACTION

    tracking_identifier_prefix = "OX"

    class Meta(OffScheduleModelMixin.Meta):
        pass


class OffSchedulePregnancy(
    ActionModelMixin, TrackingModelMixin, OffScheduleModelMixin, BaseUuidModel
):
    action_name = OFFSCHEDULE_PREGNANCY_ACTION

    tracking_identifier_prefix = "OP"

    class Meta(OffScheduleModelMixin.Meta):
        pass


class OffSchedulePostnatal(
    ActionModelMixin, TrackingModelMixin, OffScheduleModelMixin, BaseUuidModel
):
    action_name = OFFSCHEDULE_POSTNATAL_ACTION

    tracking_identifier_prefix = "ON"

    class Meta(OffScheduleModelMixin.Meta):
        pass



# Generated by Django 3.2.11 on 2022-03-04 02:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0032_auto_20220304_0501'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='ogtt2_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', max_length=15, verbose_name='In opinion of the clinician, should the OGTT be repeated?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='ogtt2_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', max_length=15, verbose_name='In opinion of the clinician, should the OGTT be repeated?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='ogtt2_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', max_length=15, verbose_name='In opinion of the clinician, should the OGTT be repeated?'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='ogtt2_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', max_length=15, verbose_name='In opinion of the clinician, should the OGTT be repeated?'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='ogtt2_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', max_length=15, verbose_name='In opinion of the clinician, should the OGTT be repeated?'),
        ),
    ]



from edc_blood_results import BLOOD_RESULTS_LIPID_ACTION
from edc_blood_results.model_mixins import (
    BloodResultsModelMixin,
    CholModelMixin,
    HdlModelMixin,
    LdlModelMixin,
    TrigModelMixin,
)
from edc_crf.crf_with_action_model_mixin import CrfWithActionModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.panels import lipids_panel
from edc_model import models as edc_models


class BloodResultsLipid(
    CrfWithActionModelMixin,
    HdlModelMixin,
    LdlModelMixin,
    TrigModelMixin,
    CholModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    edc_models.BaseUuidModel,
):
    action_name = BLOOD_RESULTS_LIPID_ACTION
    tracking_identifier_prefix = "LP"
    lab_panel = lipids_panel

    class Meta(CrfWithActionModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Blood Result: Lipids"
        verbose_name_plural = "Blood Results: Lipids"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple

from ..admin_site import meta_screening_admin
from ..forms import (
    ScreeningPartTwoForm,
    calculated_fields,
    get_part_one_fields,
    get_part_three_fields,
)
from ..models import ScreeningPartTwo
from .fieldsets import (
    get_part_one_fieldset,
    get_part_three_fieldset,
    get_part_two_fieldset,
)
from .subject_screening_admin import SubjectScreeningAdmin


def get_fieldsets():
    return (
        get_part_one_fieldset(collapse=True),
        get_part_two_fieldset(),
        get_part_three_fieldset(collapse=True),
        audit_fieldset_tuple,
    )


@admin.register(ScreeningPartTwo, site=meta_screening_admin)
class ScreeningPartTwoAdmin(SubjectScreeningAdmin):

    post_url_on_delete_name = "screening_dashboard_url"
    subject_listboard_url_name = "screening_listboard_url"

    form = ScreeningPartTwoForm

    fieldsets = get_fieldsets()

    readonly_fields = (
        *get_part_one_fields(),
        *get_part_three_fields(),
        *calculated_fields,
    )



from edc_reference import site_reference_configs

site_reference_configs.register_from_visit_schedule(
    visit_models={"edc_appointment.appointment": "meta_subject.subjectvisit"}
)

# configs = {
#     "meta_subject.education": ["household_head"],
#     "meta_subject.patienthistory": ["cd4_date", "viral_load_date"],
#     "meta_subject.medicalexpenses": ["care_before_hospital"],
#     "meta_subject.week16": ["patient_alive"],
# }
#
# for reference_name, fields in configs.items():
#     site_reference_configs.add_fields_to_config(
#         name=reference_name, fields=fields)



# Generated by Django 3.2.6 on 2021-09-11 17:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0084_auto_20210910_0234"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultsins",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultslipid",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
    ]



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_mnsi.form_validator import MnsiFormValidator

from ..models import Mnsi


class MnsiForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = MnsiFormValidator

    class Meta:
        model = Mnsi
        fields = "__all__"



from edc_constants.constants import DEAD, NONE, OTHER, UNKNOWN
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_transfer.constants import TRANSFERRED

from meta_edc.meta_version import PHASE_THREE, get_meta_version
from meta_prn.constants import LATE_EXCLUSION, OTHER_RX_DISCONTINUATION, WITHDRAWAL

list_data = {
    "meta_lists.abnormalfootappearanceobservations": [
        ("deformities", "Deformities"),
        ("dry_skin_callus", "Dry skin, callus"),
        ("infection", "Infection"),
        ("fissure", "Fissure"),
        (OTHER, "Other abnormality, please specify"),
    ],
    "meta_lists.nonadherencereasons": [
        ("forget_to_take", "I sometimes forget to take my pills"),
        ("dont_like_taking", "I don't like taking my pills"),
        ("make_me_ill", "My pills sometimes make me feel sick"),
        ("misplaced_pills", "I sometimes misplace my pills"),
        ("dont_believe_pills_help", "I don't believe my pills are helping me"),
        ("dont_believe_pills_needed", "I don't believe I need to take my pills"),
        ("not_feeling_well", "I have not been feeling well"),
        (OTHER, "Other, please specify ..."),
    ],
    "meta_lists.hypertensionmedications": [
        ("amlodipine", "Amlodipine"),
        ("atenolol", "Atenolol"),
        ("bendroflumethiazide", "Bendroflumethiazide"),
        ("bisoprolol", "Bisoprolol"),
        ("captopril", "Captopril"),
        ("diltiazem", "Diltiazem"),
        ("enalapril", "Enalapril"),
        ("eplerenone", "Eplerenone"),
        ("furosemide", "Furosemide"),
        ("hydralazine", "Hydralazine"),
        ("hydrochlothiazide", "Hydrochlothiazide"),
        ("indapamide", "Indapamide"),
        ("losartan", "Losartan"),
        ("metoprolol", "Metoprolol"),
        ("nifedipine", "Nifedipine"),
        ("perindopril", "Perindopril"),
        ("propanolol", "Propanolol"),
        ("spironolactone", "Spironolactone"),
        ("telmisartan", "Telmisartan"),
        ("torsemide", "Torsemide"),
        ("verapamil", "Verapamil"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.diabetessymptoms": [
        ("frequent_urination", "Wanting to urinate more often than usual"),
        ("excessive_thirst", "Wanting to drink water more than usual"),
        ("excessive_eating", "Wanting to eat food more than usual"),
        (OTHER, "Other, specify"),
        (NONE, "No symptoms to report"),
    ],
    "meta_lists.oiprophylaxis": [
        ("tmp_smx", "TMP/SMX"),
        ("fluconazole", "Fluconazole"),
        ("isoniazid", "Isoniazid"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.symptoms": [
        (NONE, "--No symptoms to report"),
        ("abdominal_pain_general", "Abdominal pain (General)"),
        ("abdominal_pain_right_upper_quad", "Abdominal pain (Right upper quadrant)"),
        ("blurred_vision", "Blurred vision"),
        ("diarrhoea", "Diarrhoea"),
        ("dizziness", "Dizziness"),
        ("fatigue", "Fatigue"),
        ("flatulence", "Flatulence (gas)"),
        ("headaches", "Headaches"),
        ("increased_appetite", "Increased appetite"),
        ("joint_pain", "Joint pain"),
        ("loss_of_appetite", "Loss of appetite"),
        ("muscle_cramping", "Muscle cramping"),
        ("muscle_pain", "Muscle pain"),
        ("nausea", "Nausea"),
        ("pain_feet_or_lower_limbs", "Pain in feet/lower limbs"),
        ("pounding_heartbeat", "Fast or pounding heartbeat"),
        ("rash", "Rash"),
        ("shakiness", "Shakiness"),
        ("shallow_breathing", "Fast or shallow breathing"),
        ("skin_itching", "Skin itching"),
        ("sweating", "Sweating"),
        ("swelling_feet_or_lower_limbs", "Swelling of feet/lower limbs"),
        ("unusual_sleepiness", "Unusual sleepiness"),
        ("vomiting", "Vomiting"),
        ("weakness", "Weakness"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.baselinesymptoms": [
        ("abdominal_pain_general", "Abdominal pain (General)"),
        ("cough", "Cough"),
        ("diarrhoea", "Diarrhoea"),
        ("dizziness", "Dizziness"),
        ("fatigue", "Fatigue"),
        ("headaches", "Headaches"),
        ("loss_of_appetite", "Loss of appetite"),
        ("loss_of_weight", "Loss of weight"),
        ("muscle_cramping", "Muscle cramping"),
        ("shakiness", "Shakiness"),
        ("shallow_breathing", "Fast or shallow breathing"),
        ("skin_infection", "Skin infection"),
        ("sweating", "Sweating"),
        ("unusual_sleepiness", "Unusual sleepiness"),
        ("vomiting", "Vomiting"),
        ("weakness", "Weakness"),
        (NONE, "No symptoms to report"),
        (OTHER, "Other, specify"),
    ],
    "meta_lists.arvregimens": [
        ("ABC_3TC_ATV_r", "ABC + 3TC + ATV/r"),
        ("ABC_3TC_LPV_r", "ABC + 3TC + LPV/r"),
        ("AZT_3TC_ATV_r", "AZT + 3TC + ATV/r"),
        ("AZT_3TC_EFV", "AZT + 3TC + EFV"),
        ("AZT_3TC_LPV_r", "AZT + 3TC + LPV/r"),
        ("AZT_3TC_NVP", "AZT + 3TC + NVP"),
        ("D4T_3TC_NVP", "D4T + 3TC + NVP"),
        ("DTG_ABC/3TC_ATV_r", "DTG + (ABC/3TC) + ATV/r"),
        ("TDF_3TC_ATV_r", "TDF + 3TC + ATV/r"),
        ("TDF_3TC_DTG", "TDF + 3TC + DTG"),
        ("TDF_3TC_EFV", "TDF + 3TC + EFV"),
        ("TDF_3TC_LPV_r", "TDF + 3TC + LPV/r"),
        ("TDF_FTC_ATV_r", "TDF + FTC + ATV/r"),
        ("TDF_FTC_EFV", "TDF + FTC + EFV"),
        ("TDF_FTC_LPV_r", "TDF + FTC + LPV/r"),
        ("ZDV_3TC_EFV", "ZDV + 3TC + EFV"),
        ("ZDV_3TC_NVP", "ZDV + 3TC + NVP"),
        ("ZDV_LPV_NVP", "ZDV + LPV + NVP"),
        (UNKNOWN, "Unknown"),
        (OTHER, "Other, specify ..."),
    ],
    "meta_lists.offstudyreasons": [
        ("completed_followup", "Patient completed 36 months of follow-up"),
        ("diabetes", "Patient developed diabetes"),
        ("clinical_withdrawal", "Patient is withdrawn on CLINICAL grounds ..."),
        ("", ""),
        ("clinical_endpoint", "Patient reached a clinical endpoint"),
        ("toxicity", "Patient experienced an unacceptable toxicity"),
        (
            "intercurrent_illness",
            "Intercurrent illness which prevents further treatment",
        ),
        (LOST_TO_FOLLOWUP, "Patient lost to follow-up"),
        (DEAD, "Patient reported/known to have died"),
        (WITHDRAWAL, "Patient withdrew consent to participate further"),
        (LATE_EXCLUSION, "Patient fulfilled late exclusion criteria*"),
        (TRANSFERRED, "Patient has been transferred to another health centre"),
        (
            OTHER_RX_DISCONTINUATION,
            "Other condition that justifies the discontinuation of "
            "treatment in the clinician’s opinion (specify below)",
        ),
        (
            OTHER,
            "Other reason (specify below)",
        ),
    ],
    "meta_lists.subjectvisitmissedreasons": [
        ("forgot", "Forgot / Can’t remember being told about appointment"),
        ("family_emergency", "Family emergency (e.g. funeral) and was away"),
        ("travelling", "Away travelling/visiting"),
        ("working_schooling", "Away working/schooling"),
        ("too_sick", "Too sick or weak to come to the centre"),
        ("lack_of_transport", "Transportation difficulty"),
        (OTHER, "Other reason (specify below)"),
    ],
}
if get_meta_version() == PHASE_THREE:
    # TODO: customize for META PHASE THREE. see updated EoS form for the reasons
    """
    4 = Patient experienced an unacceptable toxicity
        A = Development of lactic acidosis or hyperlactatemia
        B = Development of hepatomegaly with steatosis
        C = Other (specify below)
    """

    list_data.update(
        {
            "meta_lists.offstudyreasons": [
                ("completed_followup", "Patient completed 36 months of follow-up"),
                ("diabetes", "Patient developed diabetes"),
                ("clinical_withdrawal", "Patient is withdrawn on CLINICAL grounds ..."),
                (
                    "toxicity",
                    "Patient experienced an unacceptable toxicity, specify below ...",
                ),
                ("clinical_endpoint", "Patient reached a clinical endpoint"),
                ("toxicity", "Patient experienced an unacceptable toxicity"),
                (
                    "intercurrent_illness",
                    "Intercurrent illness which prevents further treatment",
                ),
                (LOST_TO_FOLLOWUP, "Patient lost to follow-up"),
                (DEAD, "Patient reported/known to have died"),
                (WITHDRAWAL, "Patient withdrew consent to participate further"),
                (
                    LATE_EXCLUSION,
                    (
                        "Patient fulfilled late exclusion criteria (due to abnormal "
                        "blood values or raised blood pressure at enrolment"
                    ),
                ),
                (TRANSFERRED, "Patient has been transferred to another health centre"),
                (OTHER, "Other reason (specify below)"),
            ]
        }
    )



# Generated by Django 3.2.11 on 2022-03-04 20:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0035_auto_20220304_2233'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='repeat_fasting',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending'), ('N/A', 'Not applicable')], default='N/A', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='repeat_fasting',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending'), ('N/A', 'Not applicable')], default='N/A', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='repeat_fasting',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending'), ('N/A', 'Not applicable')], default='N/A', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='repeat_fasting',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending'), ('N/A', 'Not applicable')], default='N/A', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='repeat_fasting',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending'), ('N/A', 'Not applicable')], default='N/A', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
    ]



# Generated by Django 3.2.11 on 2022-03-04 19:33

import django.core.validators
from django.db import migrations, models
import edc_model.models.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ('meta_screening', '0034_auto_20220304_0508'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalscreeningpartone',
            name='ogtt2_performed',
        ),
        migrations.RemoveField(
            model_name='historicalscreeningpartthree',
            name='ogtt2_performed',
        ),
        migrations.RemoveField(
            model_name='historicalscreeningparttwo',
            name='ogtt2_performed',
        ),
        migrations.RemoveField(
            model_name='historicalsubjectscreening',
            name='ogtt2_performed',
        ),
        migrations.RemoveField(
            model_name='subjectscreening',
            name='ogtt2_performed',
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='converted_ifg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Fasting glucose'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Blood glucose level 2-hours'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='ifg2_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='ifg2_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='ifg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='IFG units'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='ifg2_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='repeat_fasting_duration_minutes',
            field=models.IntegerField(help_text='system calculated value', null=True),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='repeat_fasting_duration_str',
            field=models.CharField(blank=True, help_text='As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc', max_length=8, null=True, validators=[django.core.validators.RegexValidator('^([0-9]{1,3}h([0-5]?[0-9]m)?)$', message='Invalid format. Expected something like 1h20m, 11h5m, etc')], verbose_name='How long have they fasted in hours and/or minutes?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='repeat_fasting_opinion',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], max_length=15, null=True, verbose_name='In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='repeat_glucose_opinion',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', help_text='If to be repeated, do so at least 3 days after the first OGTT', max_length=15, verbose_name='In opinion of the clinician, should the glucose measurements be repeated?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartone',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending')], default='No', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='converted_ifg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Fasting glucose'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Blood glucose level 2-hours'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='ifg2_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='ifg2_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='ifg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='IFG units'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='ifg2_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='repeat_fasting_duration_minutes',
            field=models.IntegerField(help_text='system calculated value', null=True),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='repeat_fasting_duration_str',
            field=models.CharField(blank=True, help_text='As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc', max_length=8, null=True, validators=[django.core.validators.RegexValidator('^([0-9]{1,3}h([0-5]?[0-9]m)?)$', message='Invalid format. Expected something like 1h20m, 11h5m, etc')], verbose_name='How long have they fasted in hours and/or minutes?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='repeat_fasting_opinion',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], max_length=15, null=True, verbose_name='In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='repeat_glucose_opinion',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', help_text='If to be repeated, do so at least 3 days after the first OGTT', max_length=15, verbose_name='In opinion of the clinician, should the glucose measurements be repeated?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningpartthree',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending')], default='No', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='converted_ifg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Fasting glucose'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Blood glucose level 2-hours'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='ifg2_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='ifg2_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='ifg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='IFG units'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='ifg2_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='repeat_fasting_duration_minutes',
            field=models.IntegerField(help_text='system calculated value', null=True),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='repeat_fasting_duration_str',
            field=models.CharField(blank=True, help_text='As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc', max_length=8, null=True, validators=[django.core.validators.RegexValidator('^([0-9]{1,3}h([0-5]?[0-9]m)?)$', message='Invalid format. Expected something like 1h20m, 11h5m, etc')], verbose_name='How long have they fasted in hours and/or minutes?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='repeat_fasting_opinion',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], max_length=15, null=True, verbose_name='In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='repeat_glucose_opinion',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', help_text='If to be repeated, do so at least 3 days after the first OGTT', max_length=15, verbose_name='In opinion of the clinician, should the glucose measurements be repeated?'),
        ),
        migrations.AddField(
            model_name='historicalscreeningparttwo',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending')], default='No', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='converted_ifg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Fasting glucose'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Blood glucose level 2-hours'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='ifg2_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='ifg2_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='ifg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='IFG units'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='ifg2_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='repeat_fasting_duration_minutes',
            field=models.IntegerField(help_text='system calculated value', null=True),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='repeat_fasting_duration_str',
            field=models.CharField(blank=True, help_text='As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc', max_length=8, null=True, validators=[django.core.validators.RegexValidator('^([0-9]{1,3}h([0-5]?[0-9]m)?)$', message='Invalid format. Expected something like 1h20m, 11h5m, etc')], verbose_name='How long have they fasted in hours and/or minutes?'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='repeat_fasting_opinion',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], max_length=15, null=True, verbose_name='In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='repeat_glucose_opinion',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', help_text='If to be repeated, do so at least 3 days after the first OGTT', max_length=15, verbose_name='In opinion of the clinician, should the glucose measurements be repeated?'),
        ),
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending')], default='No', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='converted_ifg2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Fasting glucose'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='converted_ogtt2_value',
            field=models.DecimalField(decimal_places=4, help_text='mmol/L (system converted)', max_digits=8, null=True, verbose_name='REPEAT Blood glucose level 2-hours'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='ifg2_datetime',
            field=models.DateTimeField(blank=True, null=True, verbose_name='<u>Time</u> FBG level measured'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='ifg2_quantifier',
            field=models.CharField(choices=[('=', '='), ('>', '>'), ('>=', '>='), ('<', '<'), ('<=', '<=')], default='=', max_length=10, verbose_name='FBG quantifier'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='ifg2_units',
            field=models.CharField(blank=True, choices=[('mg/dL', 'mg/dL'), ('mmol/L', 'mmol/L (millimoles/L)')], max_length=15, null=True, verbose_name='IFG units'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='ifg2_value',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='A `HIGH` reading may be entered as 9999.99', max_digits=8, null=True, verbose_name='FBG level'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='repeat_fasting',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], help_text='As reported by patient', max_length=15, null=True, verbose_name='Has the participant fasted?'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='repeat_fasting_duration_minutes',
            field=models.IntegerField(help_text='system calculated value', null=True),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='repeat_fasting_duration_str',
            field=models.CharField(blank=True, help_text='As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc', max_length=8, null=True, validators=[django.core.validators.RegexValidator('^([0-9]{1,3}h([0-5]?[0-9]m)?)$', message='Invalid format. Expected something like 1h20m, 11h5m, etc')], verbose_name='How long have they fasted in hours and/or minutes?'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='repeat_fasting_opinion',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No'), ('not_sure', 'Not sure')], max_length=15, null=True, verbose_name='In the opinion of the clinican, has the participant fasted for these repeat glucose measurement?'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='repeat_glucose_opinion',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], default='No', help_text='If to be repeated, do so at least 3 days after the first OGTT', max_length=15, verbose_name='In opinion of the clinician, should the glucose measurements be repeated?'),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='repeat_glucose_performed',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('PENDING', 'Pending')], default='No', help_text='If repeated, must be at least 3 days after the first glucose measures (FBG, OGTT)', max_length=15, verbose_name='Were the glucose measurements repeated?'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ogtt2_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ogtt2_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ogtt_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartone',
            name='ogtt_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ogtt2_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ogtt2_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ogtt_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningpartthree',
            name='ogtt_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ogtt2_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ogtt2_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ogtt_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalscreeningparttwo',
            name='ogtt_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ogtt2_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ogtt2_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ogtt_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='ogtt_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ogtt2_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ogtt2_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ogtt_base_datetime',
            field=models.DateTimeField(blank=True, help_text='(glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> oral glucose solution was given'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='ogtt_datetime',
            field=models.DateTimeField(blank=True, help_text='(2 hours after glucose solution given)', null=True, validators=[edc_model.models.validators.date.datetime_not_future], verbose_name='<u>Time</u> blood glucose measured 2-hours after oral glucose solution given'),
        ),
    ]



from bs4 import BeautifulSoup
from django import template
from django.conf import settings
from edc_constants.constants import NO, TBD, YES
from edc_dashboard.url_names import url_names

from meta_screening.eligibility import MetaEligibility

register = template.Library()


@register.inclusion_tag(
    f"meta_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/"
    f"buttons/screening_button.html",
    takes_context=True,
)
def screening_button(context, model_wrapper):
    title = "Edit subject's screening form"
    perms = context["perms"]

    p1 = model_wrapper.object.eligible_part_one

    continue_p2 = YES
    if (
        model_wrapper.object.eligible_part_one == NO
        and model_wrapper.object.continue_part_two == NO
    ):
        continue_p2 = NO

    p2 = model_wrapper.object.eligible_part_two
    p3 = model_wrapper.object.eligible_part_three
    p1_enabled = perms.user.has_perms(
        "meta_screening.view_screeningpartone"
    ) or perms.user.has_perm("meta_screening.change_screeningpartone")
    p2_enabled = (
        perms.user.has_perm("meta_screening.view_screeningparttwo")
        or perms.user.has_perm("meta_screening.change_screeningparttwo")
    ) and p1 in [YES, NO]
    p3_enabled = (
        (
            perms.user.has_perm("meta_screening.view_screeningparttwo")
            or perms.user.has_perm("meta_screening.change_screeningparttwo")
        )
        and p1 == YES
        and p2 == YES
    )
    return dict(
        continue_p2=continue_p2,
        perms=context["perms"],
        screening_identifier=model_wrapper.object.screening_identifier,
        href_p1=model_wrapper.href_p1,
        href_p2=model_wrapper.href_p2,
        href_p3=model_wrapper.href_p3,
        p1=p1,
        p2=p2,
        p3=p3,
        p1_enabled=p1_enabled,
        p2_enabled=None if continue_p2 == NO else p2_enabled,
        p3_enabled=p3_enabled,
        title=title,
        YES=YES,
        NO=NO,
        TBD=TBD,
    )


@register.inclusion_tag(
    f"meta_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/"
    f"buttons/eligibility_button.html"
)
def eligibility_button(subject_screening_model_wrapper):
    comment = []
    obj = subject_screening_model_wrapper.object
    tooltip = None
    if obj.reasons_ineligible:
        comment = obj.reasons_ineligible.split("|")
        comment = list(set(comment))
        comment.sort()
    eligibility = MetaEligibility(obj, update_model=False)
    soup = BeautifulSoup(eligibility.display_label, features="html.parser")
    return dict(
        eligible=obj.eligible,
        eligible_final=obj.eligible,
        display_label=soup.get_text(),
        comment=comment,
        tooltip=tooltip,
        TBD=TBD,
    )


@register.inclusion_tag(
    f"meta_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/buttons/add_consent_button.html",
    takes_context=True,
)
def add_consent_button(context, model_wrapper):
    title = ["Consent subject to participate."]
    consent_version = model_wrapper.consent.version
    return dict(
        perms=context["perms"],
        screening_identifier=model_wrapper.object.screening_identifier,
        href=model_wrapper.consent.href,
        consent_version=consent_version,
        title=" ".join(title),
    )


def refusal_button(context, subject_refusal_model_wrapper):
    title = ["Capture subject's primary reason for not joining."]

    return dict(
        perms=context["perms"],
        href=subject_refusal_model_wrapper.href,
        title=" ".join(title),
    )


@register.inclusion_tag(
    f"meta_dashboard/bootstrap{settings.EDC_BOOTSTRAP}/"
    f"buttons/dashboard_button.html"
)
def dashboard_button(model_wrapper):
    subject_dashboard_url = url_names.get("subject_dashboard_url")
    return dict(
        subject_dashboard_url=subject_dashboard_url,
        subject_identifier=model_wrapper.subject_identifier,
    )



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple

from ..admin_site import meta_subject_admin
from ..forms import HealthEconomicsForm
from ..models import HealthEconomics
from .modeladmin import CrfModelAdmin


@admin.register(HealthEconomics, site=meta_subject_admin)
class HealthEconomicsAdmin(CrfModelAdmin):

    form = HealthEconomicsForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Part 1: Education",
            {
                "fields": (
                    "occupation",
                    "education_in_years",
                    "education_certificate",
                    "primary_school",
                    "primary_school_in_years",
                    "secondary_school",
                    "secondary_school_in_years",
                    "higher_education",
                    "higher_education_in_years",
                )
            },
        ),
        (
            "Part 2: Household Income and Expenditures",
            {
                "fields": (
                    "welfare",
                    "income_per_month",
                    "household_income_per_month",
                    "is_highest_earner",
                    "highest_earner",
                    "food_per_month",
                    "accomodation_per_month",
                    "large_expenditure_year",
                    "buy_meds_month",
                    # "diabetic_expenditure_month",
                    # "diabetic_payee",
                    "hypertensive_expenditure_month",
                    "hypertensive_payee",
                    "arv_expenditure_month",
                    "arv_payee",
                    "meds_other_expenditure_month",
                    "meds_other_payee",
                    "expenditure_other_month",
                    "expenditure_other_detail",
                    "expenditure_other",
                    "expenditure_other_payee",
                    "healthcare_expenditure_month",
                )
            },
        ),
        (
            "Part 3: Work, Childcare, Transport",
            {
                "fields": (
                    "routine_activities",
                    "routine_activities_other",
                    "off_work_days",
                    "travel_time",
                    "hospital_time",
                    "lost_income",
                    "lost_income_amount",
                    "childcare",
                    "childcare_source",
                    "childcare_source_timeoff",
                    "transport_old",
                    "transport_cost",
                    "transport_barter",
                    "transport_borrow",
                    "health_insurance",
                    "health_insurance_pay",
                )
            },
        ),
        audit_fieldset_tuple,
    )

    radio_fields = {
        "primary_school": admin.VERTICAL,
        "secondary_school": admin.VERTICAL,
        "higher_education": admin.VERTICAL,
        "welfare": admin.VERTICAL,
        "is_highest_earner": admin.VERTICAL,
        "buy_meds_month": admin.VERTICAL,
        "diabetic_payee": admin.VERTICAL,
        "hypertensive_payee": admin.VERTICAL,
        "arv_payee": admin.VERTICAL,
        "meds_other_payee": admin.VERTICAL,
        "expenditure_other_payee": admin.VERTICAL,
        "routine_activities": admin.VERTICAL,
        "lost_income": admin.VERTICAL,
        "childcare": admin.VERTICAL,
        "childcare_source": admin.VERTICAL,
        "transport_old": admin.VERTICAL,
        "transport_barter": admin.VERTICAL,
        "transport_borrow": admin.VERTICAL,
        "health_insurance": admin.VERTICAL,
        "expenditure_other_month": admin.VERTICAL,
    }



from edc_constants.constants import CLOSED, OTHER, YES
from edc_form_validators import FormValidator

from ..constants import VIOLATION


class ProtocolDeviationViolationFormValidator(FormValidator):
    def clean(self):

        self.applicable_if(
            VIOLATION, field="report_type", field_applicable="safety_impact"
        )

        self.applicable_if(
            VIOLATION, field="report_type", field_applicable="study_outcomes_impact"
        )

        self.required_if(
            YES, field="safety_impact", field_required="safety_impact_details"
        )

        self.required_if(
            YES,
            field="study_outcomes_impact",
            field_required="study_outcomes_impact_details",
        )

        self.required_if(
            VIOLATION, field="report_type", field_required="violation_datetime"
        )

        self.applicable_if(
            VIOLATION, field="report_type", field_applicable="violation_type"
        )

        self.validate_other_specify(
            field="violation_type",
            other_specify_field="violation_type_other",
            other_stored_value=OTHER,
        )

        self.required_if(
            VIOLATION, field="report_type", field_required="violation_description"
        )

        self.required_if(
            VIOLATION, field="report_type", field_required="violation_reason"
        )

        self.required_if_not_none(
            field="corrective_action_datetime", field_required="corrective_action"
        )

        self.required_if_not_none(
            field="preventative_action_datetime", field_required="preventative_action"
        )

        self.required_if(
            CLOSED, field="report_status", field_required="report_closed_datetime"
        )



from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from edc_action_item.models import ActionItem
from edc_appointment.models import Appointment
from edc_constants.constants import FEMALE
from edc_utils import get_utcnow
from edc_visit_schedule.constants import MONTH1
from model_bakery.baker import make_recipe

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_visit_schedule.constants import DELIVERY


class TestMetadataRules(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(gender=FEMALE)
        self.subject_consent = self.get_subject_consent(self.subject_screening)
        self.subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )
        self.data = dict(
            subject_visit=self.subject_visit.pk,
            report_datetime=self.subject_visit.report_datetime,
        )

    def test_pregnancy_actions(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)

        urine_pregnancy = make_recipe(
            "meta_subject.urinepregnancy",
            subject_visit=subject_visit,
            report_datetime=get_utcnow(),
            assay_date=get_utcnow().date(),
        )

        try:
            ActionItem.objects.get(
                action_identifier=urine_pregnancy.action_identifier,
                reference_model="meta_subject.urinepregnancy",
            )
        except ObjectDoesNotExist:
            self.fail("ActionItem for urinepregnancy unexpectedly does not exist")

        try:
            ActionItem.objects.get(
                parent_action_item__action_identifier=urine_pregnancy.action_identifier,
                reference_model="meta_prn.pregnancynotification",
            )
        except ObjectDoesNotExist:
            self.fail(
                "ActionItem for pregnancynotification unexpectedly does not exist"
            )

        make_recipe(
            "meta_prn.pregnancynotification",
            subject_identifier=subject_visit.subject_identifier,
        )

        try:
            Appointment.objects.get(visit_code=DELIVERY)
        except ObjectDoesNotExist:
            self.fail("delivery appointment unexpectedly does not exist")

        subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=DELIVERY,
            visit_code_sequence=0,
        )

        delivery = make_recipe(
            "meta_subject.delivery",
            subject_visit=subject_visit,
        )

        try:
            ActionItem.objects.get(
                parent_action_item__action_identifier=delivery.action_identifier,
                reference_model="meta_prn.offschedulepregnancy",
            )
        except ObjectDoesNotExist:
            self.fail("ActionItem for offschedulepregnancy unexpectedly does not exist")



from edc_model import models as edc_models
from edc_qol.model_mixins import Eq5d3lModelMixin

from ..model_mixins import CrfModelMixin


class Eq5d3l(
    Eq5d3lModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):
    class Meta(
        Eq5d3lModelMixin.Meta, CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta
    ):
        pass
